function [V_best,cost_best,G_best,n_tot]=mpfa_cnstr(fhandle,Lb,Ub,Para,...
                                                    Data,info)
% Main function for constrained optimization
%
% Functions:
%     mpfa_crsp.m
%     mpfa_alpha.m
%     mpfa_fun.m
%     mpfa_sort.m
%     mpfa_rcsde.m
%     mpfa_move.m
% _________________________________________________________________________

% Check whether the upper bound & lower bound are the same size
if length(Lb) ~= length(Ub),
    disp(' Incorrectly defined limits for variables ');
    V_best    = [];
    cost_best = [];
    G_best    = [];
    n_tot     = [];
    return
else
    d = length(Lb); % Calculate dimension
end

% Parameters
n       = Para(1);
np      = Para(2);
nmax    = Para(3);
alpha   = Para(4);
rdm     = Para(5);
speed   = Para(6);
betamin = Para(7);
gamma   = Para(8);
m       = Para(9);
lsrd    = Para(10);
gsrd    = Para(11);
 
% Initial values for arrays, vectors and scalars
VO       = ones(n,d,np)*10^100; % Optimization variables
FIT_FUN  = ones(n,1,np)*10^100; % Fitness function (ff)
COST     = ones(n,1,np)*10^100; % Cost
PHI      = ones(n,1,np)*10^100; % Strength reduction factor (ACI318)
ROT      = ones(n,1,np)*10^100; % Section gyration
G        = ones(n,14,np)*10^100;% Constraints
COS_PHI  = ones(n,1,np)*10^100; % Cosine of angle (2nd strength constraint)
COSTOC   = ones(n,1,np)*10^100; % Cost of the Best Design              (BD)
PHIC     = ones(n,1,np)*10^100; % Strength reduction factor (ACI318)   (BD)
ROTC     = ones(n,1,np)*10^100; % Section gyration                     (BD)
GC       = ones(n,14,np)*10^100;% Constraints                          (BD)
COS_PHIC = ones(n,1,np)*10^100; % Cosine of angle (2nd strength const.)(BD)
N_np     = zeros(1,np);         % Number of ff evaluations in each subpop.
n_tot    = 0;                   % Total number of ff evaluations

% Initial locations of n fireflies in each subpopulation
parfor j = 1 : np
    [VO(:,:,j)] = mpfa_crsp(n,Lb,Ub);
end

% Arrays for graphic output
FBEST      = zeros(1,nmax);
FFBEST     = zeros(1,nmax);
FMEAN      = zeros(1,nmax);
ITERATIONS = zeros(1,nmax);

% Main for-loop
n_fiber_min = 100; % minimum number of fibers to consider in mpfa_rcsde.m
n_fiber_max = 200; % maximum number of fibers to consider in mpfa_rcsde.m
for k = 1 : nmax
%   the number of fibers increases linearly from n_fiber_min to
%   n_fiber_max as k increases from 1 to nmax (so the precision increases)
    n_fiber = ( 1 / (nmax- 1) ) * ( k * ( n_fiber_max - n_fiber_min ) + ...
                                    n_fiber_min * nmax - n_fiber_max );
    n_fiber = round(n_fiber); % num. of fibers to consider in mpfa_rcsde.m

%   Quadratic reduction of the alpha parameter
    if k == 1, alpha0 = alpha; end
    alpha = mpfa_alpha(k, nmax, rdm, speed) * alpha0;

%   Quadratic reduction of the gamma parameter    
    if k == 1, gamma0 = gamma; end
    gamma = mpfa_alpha(k, nmax, rdm, speed) * gamma0;

%   Fitness function calculated for n fireflies in each subpopulation
    parfor j = 1 : np
        [FIT_FUN(:,1,j), COST(:,1,j), PHI(:,1,j), ...
         ROT(:,1,j), G(:,:,j), COS_PHI(:,1,j)] = ...
            mpfa_fun(fhandle, n, VO(:,:,j), n_fiber, Data);
        N_np(1,j) = N_np(1,j) + n;
    end

%   Rank fireflies by their fitness
    parfor j = 1 : np
        [FIT_FUN(:,1,j), VO(:,:,j), COSTOC(:,:,j), ...
         PHIC(:,:,j), ROTC(:,:,j), GC(:,:,j), COS_PHIC(:,:,j)] = ...
            mpfa_sort(n, FIT_FUN(:,1,j), VO(:,:,j), COST(:,1,j), ...
                     PHI(:,1,j), ROT(:,1,j), G(:,:,j), COS_PHI(:,1,j));
    end

%   Find the current best design
    [~,Index]    = sort(FIT_FUN(1,1,:),3);
    ff_best      = FIT_FUN(1,1,Index(1));
    cost_best    = COSTOC(1,1,Index(1));
    phi_best     = PHIC(1,1,Index(1));
    rot_best     = ROTC(1,1,Index(1));
    G_best       = GC(1,:,Index(1));
    cos_phi_best = COS_PHIC(1,1,Index(1));
    f_best       = tan( (pi ./ 2) .* VO(1,1,Index(1)) ) .* 1000;        
    
    if d == 7
        Max          = Data(1,19:24);
        teta_best    = VO(1,2,Index(1)).* Max(1); 
        b_best       = VO(1,3,Index(1)).* Max(2);
        h_best       = VO(1,4,Index(1)).* Max(3);
        phi_py_best  = VO(1,5,Index(1)).* Max(4);
        n_py_best    = VO(1,6,Index(1)).* Max(5);
        phi_y_best   = VO(1,5,Index(1)).* Max(4);
        n_y_best     = VO(1,6,Index(1)).* Max(5);
        phi_px_best  = VO(1,5,Index(1)).* Max(4);
        n_px_best    = VO(1,7,Index(1)).* Max(6);
        phi_x_best   = VO(1,5,Index(1)).* Max(4);
        n_x_best     = VO(1,7,Index(1)).* Max(6);
    elseif d == 8
        Max          = Data(1,19:25);
        teta_best    = VO(1,2,Index(1)).* Max(1); 
        b_best       = VO(1,3,Index(1)).* Max(2);
        h_best       = VO(1,4,Index(1)).* Max(3);
        phi_py_best  = VO(1,5,Index(1)).* Max(4);
        n_py_best    = VO(1,6,Index(1)).* Max(5);
        phi_y_best   = VO(1,5,Index(1)).* Max(4);
        n_y_best     = VO(1,6,Index(1)).* Max(5);
        phi_px_best  = VO(1,7,Index(1)).* Max(6);
        n_px_best    = VO(1,8,Index(1)).* Max(7);
        phi_x_best   = VO(1,7,Index(1)).* Max(6);
        n_x_best     = VO(1,8,Index(1)).* Max(7);    
    elseif d == 12
        Max          = Data(1,19:29);
        teta_best    = VO(1,2,Index(1)).* Max(1); 
        b_best       = VO(1,3,Index(1)).* Max(2);
        h_best       = VO(1,4,Index(1)).* Max(3);
        phi_py_best  = VO(1,5,Index(1)).* Max(4);
        n_py_best    = VO(1,6,Index(1)).* Max(5);
        phi_y_best   = VO(1,7,Index(1)).* Max(6);
        n_y_best     = VO(1,8,Index(1)).* Max(7);
        phi_px_best  = VO(1,9,Index(1)).* Max(8);
        n_px_best    = VO(1,10,Index(1)).* Max(9);
        phi_x_best   = VO(1,11,Index(1)).* Max(10);
        n_x_best     = VO(1,12,Index(1)).* Max(11);
    end
    
    V_best= [f_best teta_best b_best h_best ...
             phi_py_best n_py_best phi_y_best n_y_best ...
             phi_px_best n_px_best phi_x_best n_x_best]';   
    
    if info % display the evolution of the design
        f_info        = tan((pi./2).*VO(:,1,:)).*1000;
        teta_info     = VO(:,2,:) .* Max(1);
        b_info        = VO(:,3,:) .* Max(2);
        h_info        = VO(:,4,:) .* Max(3);
        
        FBEST(k)      = cost_best;           
        FFBEST(k)     = ff_best;
        ITERATIONS(k) = k;
        
        FMEAN(k) = mean(FIT_FUN(:,1,Index(1)));

        figure(1)
        pos1 = [0.05 0.50 0.30 0.40];
        set(gcf,'numbertitle','off','Units','normalized','Position',...
        pos1,'color','white')
        subplot(2,1,1)
        plot(ITERATIONS(1,1:k), FFBEST(1,1:k), '-b','Linewidth',1)
        ylabel('Fitness Function')
        subplot(2,1,2)
        plot(ITERATIONS(1,1:k), FBEST(1,1:k), '-r','Linewidth',1)
        xlabel('Iterations')
        ylabel('Cost (Euro/m)')

        if Lb(3)*Max(2) ~= Max(2)
            figure(2)
            pos2 = [0.40 0.55 0.35 0.35];
            set(gcf,'numbertitle','off','Units','normalized','Position',...
                pos2,'color','white')             
            for j = 1 : np
               ccolor = [ (np-j+1)/(np+1) 0.5 j/(np+1) -fix(j/(np+1)) ];
               plot3(b_info(:,:,j), h_info(:,:,j), teta_info(:,:,j),...
                     '.','Color',ccolor)
               hold on
            end    
            plot3(b_best, h_best, teta_best, 'or','LineWidth',2)
            hold off
            axis([Lb(3)*Max(2) Max(2) Lb(4)*Max(3) Max(3) Lb(2)*Max(1) Max(1)])
            xlabel('b (mm)')
            ylabel('h (mm)')
            zlabel('teta (rad)')
            grid on
         else
            figure(2)
            pos2=[0.40 0.55 0.35 0.35];
            set(gcf,'numbertitle','off','Units','normalized','Position',...
            pos2,'color','white')     
            for j=1:np
               plot(f_info(:,:,j),teta_info(:,:,j),'.b')
            hold on
            end  
            plot(f_best, teta_best,'or')
            hold off
            axis([tan((pi./2).*Lb(1)).*1000 tan((pi./2).*Ub(1)).*1000 ...
                  Lb(2)*Max(1) Max(1)])
            xlabel('f (1st var.)')
            ylabel('teta (2nd var.)')
            grid on
        end   
        figure(3)
        pos3 = [0.05 0.08 0.30 0.30];
        set(gcf,'numbertitle','off','Units','normalized','Position',...
        pos3,'color','white')
        plot(COS_PHI(:,:,Index(1)), G(:,5,Index(1)), ...
             '.b',cos_phi_best,G_best(1,5),'or')
        axis([-1 1 -1 0])
        xlabel('cos(\psi)')
        ylabel('g_s_2')
        grid on
        drawnow
        disp(['_______________________________________________________' ...
              '_____________________________________'])
        disp(['f = ',     num2str(f_best),      ' mm; ',      ...
              'theta = ', num2str(teta_best),   ' rad; ',     ...
              'b = ',     num2str(b_best),      ' mm; ',      ...
              'h = ',     num2str(h_best),      ' mm; '])
        disp(['phi_py = ',num2str(phi_py_best), ' mm; ',      ...
              'n_py = ',  num2str(n_py_best),   ' u; ',       ...
              'phi_y = ', num2str(phi_y_best),  ' mm; ',      ...
              'n_y = ',   num2str(n_y_best),    ' u; '])
        disp(['phi_px = ',num2str(phi_px_best), ' mm; ',      ...
              'n_px = ',  num2str(n_px_best),   ' u; ',       ...
              'phi_x = ', num2str(phi_x_best),  ' mm; ',      ...
              'n_x = ',   num2str(n_x_best),    ' u; '])
        disp(['Cost   = ',num2str(cost_best),   ' Euro/m;'])  
        disp(['Section gyration  = ', num2str(rot_best * 1e6),   ...
                                                ' (10-3 m-1);'])          
        As_py = (pi/4) .* floor(n_py_best) .* phi_py_best .^2;
        As_y  = (pi/4) .* floor(n_y_best)  .* phi_y_best  .^2;
        As_px = (pi/4) .* floor(n_px_best) .* phi_px_best .^2;
        As_x  = (pi/4) .* floor(n_x_best)  .* phi_x_best  .^2;
        disp(['Left reinforcement area   = ',   num2str(As_y),  ' mm2;'])          
        disp(['Right reinforcement area  = ',   num2str(As_py), ' mm2;'])          
        disp(['Top reinforcement area    = ',   num2str(As_px), ' mm2;'])          
        disp(['Bottom reinforcement area = ',   num2str(As_x),  ' mm2;'])          
        disp(['Total reinforcement area  = ', ...
               num2str(As_py + As_y + As_px + As_x),            ' mm2;'])          
        
%       Strength of the best design
        [Nn,Mnx,Mny] = mpfa_rcsde(n_fiber,                    ...
                                  f_best,      teta_best,     ...
                                  b_best,      h_best,        ...
                                  phi_y_best,  n_y_best,      ...
                                  phi_py_best, n_py_best,     ...
                                  phi_x_best,  n_x_best,      ...
                                  phi_px_best, n_px_best,     ...
                                  Data(1,11),  Data(1,12),    ...
                                  Data(1,8),   Data(1,9),     ...
                                  Data(1,15));
        
        NRd  = phi_best * Nn  /1e3; % NRd  (axial strength, kN)
        MxRd = phi_best * Mnx /1e6; % MxRd (flexural strng., x axis, kN x m)
        MyRd = phi_best * Mny /1e6; % MyRd (flexural strng., y axis, kN x m)
        disp(['NRd (kN)      = ', num2str(NRd),  '  -----> ', ...
              num2str(Data(1,5))])  
        disp(['MxRd (kN x m) = ', num2str(MxRd), '  -----> ', ... 
              num2str(Data(1,6))])              
        disp(['MyRd (kN x m) = ', num2str(MyRd), '  -----> ', ...
              num2str(Data(1,7))])              
        disp(['g_As = ',           num2str(G_best(1)),   ...
              '; g_Asp = ',        num2str(G_best(2)),   ...
              '; g_Ast = ',        num2str(G_best(3)),   ...
              '; g_duct = ',       num2str(G_best(4)),   ...
              '; g_s2 = ',         num2str(G_best(5)),   ...
              '; g_s1 = ',         num2str(G_best(6))])  
        disp(['g_esp_max_x = ',    num2str(G_best(7)),   ...
              '; g_esp_max_px = ', num2str(G_best(8)),   ...
              '; g_esp_max_y = ',  num2str(G_best(9)),  ...
              '; g_esp_max_py = ', num2str(G_best(10))])
        disp(['g_esp_min_x = ',    num2str(G_best(11)),  ...
              '; g_esp_min_px = ', num2str(G_best(12)),  ...
              '; g_esp_min_y = ',  num2str(G_best(13)),  ...
              '; g_esp_min_py = ', num2str(G_best(14))])
    end

%   Local Small Random Displacement (lsrd)
    if lsrd % if lsrd == 1 ---> fp ~= 0
        n_max = 30;
        n_min = 20;
        if n >= n_max
%           lsrd has an important effect on subpopulations where n >= 30
            fp = 1; 
        elseif n > n_min
%           linear reduction of lsrd            
            fp = (n - n_min) ./ (n_max - n_min);
        else
%           lsrd has a negligible effect in subpopulations where n <= 20              
            fp = 0;
        end
    else
        fp=0;
    end

%   Move the fireflies to the best locations in each sub-population
    beta0 = 1;
    exp_alpha = 2.0;
    parfor j = 1 : np
        [VO(:,:,j), FIT_FUN(:,1,j), N_np(1,j)]=...
            mpfa_move(fhandle, n, VO(:,:,j), FIT_FUN(:,1,j), alpha, ...
                      beta0, betamin, gamma, m, Lb, Ub, Data, ...
                      n_fiber, exp_alpha, fp);
    end
     
    for j = 1 : np   % total number of evaluations of the fitness function
        n_tot = n_tot + N_np(1,j);
    end
    
    if np > 1
        [~, Index] = sort(FIT_FUN(1,1,:), 3);
        if gsrd % apply Global Small Random Displacement
            VO(1,1,Index(1)) = VO(1,1,Index(1)) +...
              (alpha .^ exp_alpha) .* (rand(1,1) - 0.5) .*abs(Ub(1)-Lb(1));
            VO(1,2,Index(1)) = VO(1,2,Index(1)) +...
              (alpha .^ exp_alpha) .* (rand(1,1) - 0.5) .*abs(Ub(2)-Lb(2));
        end

%       Fireflies' migration among subpopulations
        startmgr = 0.0; % starting point of migration within the interval [0,1]
        if k > round(startmgr * nmax)
            for jloop = 2 : randi(np-1) : np   % choose the target population
%                                                randomly 
                for iloop = 1 : randi(n-1) : n % choose the target individual 
%                                                randomly 
                   VO(iloop,:,Index(jloop)) = VO(iloop,:,Index(1));    
                end
            end
        end
    end
    
end   % end of main for-loop

% End of parallel process
poolobj = gcp('nocreate');
delete(poolobj);

end
