function [Vo] = nb_dsc(Vo, Data)
% New values for optimization variables 
% that are linked to some discrete design variables
% (Number of Reinforcement Bars, NRB)
%
% Variables:
%     1   2    3  4    5      6     7      8     9      10    11     12
%     f  teta  b  h  phi_py  n_py  phi_y  n_y  phi_px  n_px  phi_x  n_x
% _________________________________________________________________________

    if length(Data) == 24
%              1 2 3 4 5 6 7
%                        _ _  
        Var = [0 0 0 0 0 6 7];
        Max = Data(1,19:24);
    elseif length(Data) == 25
%              1 2 3 4 5 6 7 8
%                        _   _  
        Var = [0 0 0 0 0 6 0 8];
        Max = Data(1,19:25);    
    elseif length(Data) == 29
%              1 2 3 4 5 6 7 8 9 10 11 12
%                        _   _   __    __
        Var = [0 0 0 0 0 6 0 8 0 10  0 12];
        Max = Data(1,19:29);        
    end

    Vd = zeros(1,length(Var));
    for iloop = 1 : length(Var)
        if Var(iloop) ~= 0 % it only works on variables linked to NRB
            Vd(iloop) = Vo(iloop) .* Max(iloop-1); % design variable Vd
            V_l = fix(Vd(iloop));   % minimum discrete value for Vd
            V_u = ceil(Vd(iloop));  % maximum discrete value for Vd
            Vd(iloop) = randi([V_l V_u],1); % random assignment
            Vo(iloop) = Vd(iloop) ./ Max(iloop-1); % optimiz. variable Vo
        end
    end
end
